<?php

/**
 * Fired during plugin activation
 *
 * @link       https://github.com/lelinhtinh
 * @since      1.0.0
 *
 * @package    Coupon
 * @subpackage Coupon/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @todo This should probably be in one class together with Deactivator Class.
 * @since      1.0.0
 * @package    Coupon
 * @subpackage Coupon/includes
 * @author     lelinhtinh <lelinhtinh2013@gmail.com>
 */
class Coupon_Activator
{
	/**
	 * The $_REQUEST during plugin activation.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      array    $request    The $_REQUEST array during plugin activation.
	 */
	private static $request = [];

	/**
	 * The $_REQUEST['plugin'] during plugin activation.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin    The $_REQUEST['plugin'] value during plugin activation.
	 */
	private static $plugin = COUPON_BASE_NAME;
	private static $version = COUPON_VERSION;

	/**
	 * Activate the plugin.
	 *
	 * Checks if the plugin was (safely) activated.
	 * Place to add any custom action during plugin activation.
	 *
	 * @since    1.0.0
	 */
	public static function activate()
	{
		if (
			false === self::get_request()
			|| false === self::validate_request(self::$plugin)
			|| false === self::check_caps()
		) {
			if (isset($_REQUEST['plugin'])) {
				if (!check_admin_referer('activate-plugin_' . self::$request['plugin'])) {
					exit;
				}
			} elseif (isset($_REQUEST['checked'])) {
				if (!check_admin_referer('bulk-plugins')) {
					exit;
				}
			}
		}

		/**
		 * The plugin is now safely activated.
		 * Perform your activation actions here.
		 */
		global $wpdb;
		$table_name = $wpdb->prefix . self::$plugin . 's';
		$charset_collate = $wpdb->get_charset_collate();

		$sql = "";
		if ($wpdb->get_var("SHOW TABLES LIKE '{$table_name}'") !== $table_name) {
			$sql .= "CREATE TABLE {$table_name} (
				`ID` BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
				`code` VARCHAR(16) NOT NULL,
				`type` ENUM('percentage','numeric') NOT NULL DEFAULT 'percentage',
				`value` INT(10) UNSIGNED NULL DEFAULT NULL,
				`limit` INT(10) UNSIGNED NOT NULL DEFAULT '1',
				`activated_at` TIMESTAMP NULL DEFAULT NULL,
				`expired_at` TIMESTAMP NULL DEFAULT NULL,
				`created_by` BIGINT UNSIGNED NOT NULL,
				`active` TINYINT(3) UNSIGNED NULL DEFAULT '1',
				PRIMARY KEY (`ID`),
				UNIQUE INDEX `code` (`code`),
				INDEX `active` (`active`),
				INDEX `created_by` (`created_by`)
    		) $charset_collate;";
		}
		if ($wpdb->get_var("SHOW TABLES LIKE '{$table_name}_user'") !== "{$table_name}_user") {
			$sql .= "CREATE TABLE {$table_name}_user (
				`oms_coupon_id` BIGINT UNSIGNED NOT NULL,
				`user_id` BIGINT UNSIGNED NOT NULL,
				`saved_at` TIMESTAMP NULL DEFAULT NULL,
				`active` TINYINT(3) UNSIGNED NULL DEFAULT '1',
				PRIMARY KEY (`oms_coupon_id`, `user_id`),
				INDEX `active` (`active`)
    		) $charset_collate;";
		}
		if (!empty($sql)) {
			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($sql);
		}
		add_option(self::$plugin . '_version', self::$version);
	}

	/**
	 * Get the request.
	 *
	 * Gets the $_REQUEST array and checks if necessary keys are set.
	 * Populates self::request with necessary and sanitized values.
	 *
	 * @since    1.0.0
	 * @return bool|array false or self::$request array.
	 */
	private static function get_request()
	{
		if (
			!empty($_REQUEST)
			&& isset($_REQUEST['_wpnonce'])
			&& isset($_REQUEST['action'])
		) {
			if (isset($_REQUEST['plugin'])) {
				if (false !== wp_verify_nonce(sanitize_text_field(wp_unslash($_REQUEST['_wpnonce'])), 'activate-plugin_' . sanitize_text_field(wp_unslash($_REQUEST['plugin'])))) {

					self::$request['plugin'] = sanitize_text_field(wp_unslash($_REQUEST['plugin']));
					self::$request['action'] = sanitize_text_field(wp_unslash($_REQUEST['action']));

					return self::$request;
				}
			} elseif (isset($_REQUEST['checked'])) {
				if (false !== wp_verify_nonce(sanitize_text_field(wp_unslash($_REQUEST['_wpnonce'])), 'bulk-plugins')) {

					self::$request['action'] = sanitize_text_field(wp_unslash($_REQUEST['action']));
					self::$request['plugins'] = array_map('sanitize_text_field', wp_unslash($_REQUEST['checked']));

					return self::$request;
				}
			}
		} else {

			return false;
		}
	}

	/**
	 * Validate the Request data.
	 *
	 * Validates the $_REQUESTed data is matching this plugin and action.
	 *
	 * @since    1.0.0
	 * @param string $plugin The Plugin folder/name.php.
	 * @return bool false if either plugin or action does not match, else true.
	 */
	private static function validate_request($plugin)
	{
		if (
			isset(self::$request['plugin'])
			&& $plugin === self::$request['plugin']
			&& 'activate' === self::$request['action']
		) {

			return true;
		} elseif (
			isset(self::$request['plugins'])
			&& 'activate-selected' === self::$request['action']
			&& in_array($plugin, self::$request['plugins'])
		) {
			return true;
		}

		return false;
	}

	/**
	 * Check Capabilities.
	 *
	 * We want no one else but users with activate_plugins or above to be able to active this plugin.
	 *
	 * @since    1.0.0
	 * @return bool false if no caps, else true.
	 */
	private static function check_caps()
	{
		if (current_user_can('activate_plugins')) {
			return true;
		}

		return false;
	}
}
